/*
 * @(#)CSV2SRS.java
 *
 * Summary: Converts a 2-column CSV file to a guts of a Funduc Search Replace SRS Script.
 *
 * Copyright: (c) 2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  3.4 2010-12-03 - add CSV2SRS
 */
package com.mindprod.csv;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import static java.lang.System.err;

/**
 * Converts a 2-column CSV file to a guts of a Funduc Search Replace SRS Script.
 * <p/>
 * Use: java.exe com.mindprod.CSV2SRS pairs.csv
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.4 2010-12-03 - add CSV2SRS.
 * @since 2008-05-28
 */
public final class CSV2SRS
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * Constructor also does work of converting
     *
     * @param file          CSV file to be packed to remove excess space and quotes.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab for the output file.  It is tab for the input file.
     *                      Note this is a 'char' not a "string".
     * @param quoteChar     character used to quote fields containing awkward chars.
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *                      Note this is a 'char' not a "string". For output file. No comment presumed in input file.
     *
     * @throws java.io.IOException if problems reading/writing file
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public CSV2SRS( File file, char separatorChar, char quoteChar, char commentChar ) throws IOException
        {
        // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
        CSVReader in = new CSVReader( new BufferedReader( new FileReader( file ) ), ',', '\"', "" /* no comment chars */, true, true, true );

        String srsFilename = file.getAbsolutePath();
        // replace .csv with .srs
        srsFilename = srsFilename.substring( 0, srsFilename.length() - 4 ) + ".srs";
        File srsFile = new File( srsFilename );

        PrintWriter srs = new PrintWriter( new BufferedWriter( new FileWriter( srsFile ) ) );
        try
            {
            while ( true )
                {
                final String from = in.get();
                final String to = in.get();
                in.skipToNextLine();
                srs.println( "[Search]" );  //[Search /x] would give a case insensitive replace /w would do a whole word replace.
                srs.println( from );
                srs.println( "[Replace]" );
                srs.println( to );
                }
            }
        catch ( EOFException e )
            {
            in.close();
            srs.close();
            }
        }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to CSVPack. Packs one csv file whose name is on the command line. Must have
     * extension .csv <br> Use java com.mindprod.CSVPack somefile.csv .  You can use CVTab2Comm constructor
     * in your own programs.
     *
     * @param args name of csv file to remove excess quotes and space
     */
    public static void main( String[] args )
        {
        if ( args.length != 1 )
            {
            throw new IllegalArgumentException(
                    "CSV2SRS needs a single pairs.csv on the command line. Output will be in pairs.srs." );
            }
        String filename = args[ 0 ];
        if ( !filename.endsWith( ".csv" ) )
            {
            throw new IllegalArgumentException(
                    "Bad Extension\nCSV2SRS needs a single pairs.csv on the command line. Output will be in pairs.srs." );
            }
        try
            {
            // file,  separatorChar,  quoteChar,  commentChar
            new CSV2SRS( new File( filename ), ',', '\"', '#' );
            }

        catch ( IOException e )
            {
            err.println( "CSV2SRS failed to convert " + filename );
            err.println( e.getMessage() );
            e.printStackTrace();
            }
        }// end main
    }
